<?php
/**
 * Plugin Name: Alternate SMTP By Brainvire
 * Plugin URI:  https://www.brainvire.com/
 * Description: This plugin will work when WordPress SMTP fails to send emails.
 * Version:     1.0.0
 * Author:      Brainvire
 * Author URI:  http://www.brainvire.com/
 * License:     GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: alternate-smtp
 *
 * @package alternate-smtp
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

// use PHPMailer\PHPMailer\PHPMailer;
// use PHPMailer\PHPMailer\SMTP;
// use PHPMailer\PHPMailer\Exception;

// Define the plugin constants.
define( 'BVSMTP_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'BVSMTP_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );

// Include other necessary files.
include_once 'inc/functions.php';
include_once 'inc/bvsmtp-enqueue-scripts.php';
include_once 'inc/bvsmtp-ajax-functions.php';
include_once 'inc/bvsmtp-admin-pages.php';

add_action( 'wp_mail_failed', 'bvsmtp_on_smtp_mail_failed', 1, 1 );

/**
 * Try alternate smtp settings to send mail on main smtp fails.
 *
 * @param WP_Error $wp_error class WP_Error's object.
 * @return void
 */
function bvsmtp_on_smtp_mail_failed( $wp_error ) {

	if ( ! empty( $wp_error->error_data['wp_mail_failed']['to'] ) ) {
		//$error_data = array();
		$error_data = $wp_error->error_data['wp_mail_failed'];

		$send_to = array();
		$subject = ! empty( $error_data['subject'] ) ? sanitize_text_field( $error_data['subject'] ) : '';
		$message = ! empty( $error_data['message'] ) ? wp_kses_post( $error_data['message'] ) : '';


		if ( ! empty( $error_data['to'] ) ) {
			foreach ( $error_data['to'] as $mail_to ) {
				$send_to[] = sanitize_email( $mail_to );
			}
		}

		if ( function_exists( 'bvsmtp_send_mail_using_custom_smtp' ) ) {
			bvsmtp_send_mail_using_custom_smtp( $send_to, $subject, $message );
		}
	}

}

/**
 * Send mail to admin using alternate smtp settings on main smtp fails or down.
 *
 * @param array  $working_smtp Working SMTP Setting.
 * @param string $failed_smtp Failed SMTP Settings.
 * @return void
 */
function bvsmtp_send_default_admin_mail_about_smtp_down( $working_smtp, $failed_smtp = array() ) {
	$send_to = array();

	$bvsmtp_smpt_global_settings = get_option( 'bvsmtp_smpt_global_settings' );

	if ( ! empty( $bvsmtp_smpt_global_settings ) && is_array( $bvsmtp_smpt_global_settings ) ) {

		$smtp_downtime_sendmailto = ! empty( $bvsmtp_smpt_global_settings['smtp_downtime_sendmailto'] ) ? sanitize_email( $bvsmtp_smpt_global_settings['smtp_downtime_sendmailto'] ) : '';

		if ( ! empty( $smtp_downtime_sendmailto ) ) {
			$smtp_downtime_status = ! empty( $bvsmtp_smpt_global_settings['smtp_downtime_status'] ) && 'disabled' == $bvsmtp_smpt_global_settings['smtp_downtime_status'] ? false : true;

			if ( ! $smtp_downtime_status ) {
				return; // Don't send downtime mail notification as it is disabled.
			}

			$smtp_downtime_interval = ! empty( $bvsmtp_smpt_global_settings['smtp_downtime_interval'] ) ? (int) ( $bvsmtp_smpt_global_settings['smtp_downtime_interval'] * 3600 ) : ( 24 * 3600 );
			$smtp_downtime_last_send_datetime = ! empty( $bvsmtp_smpt_global_settings['smtp_downtime_last_send_datetime'] ) ? $bvsmtp_smpt_global_settings['smtp_downtime_last_send_datetime'] : '';

			if ( ! empty( $smtp_downtime_last_send_datetime ) ) {
				if ( ( abs( time() - strtotime( $smtp_downtime_last_send_datetime ) ) ) < $smtp_downtime_interval ) {
					return; // Don't send downtime mail notification as downtime interval is greater.
				}
			}

			$subject = 'ATTENTION! MAIN SMTP DOWN...';
			$message = 'MAIN SMTP IS DOWN FOR THE SITE: ' . esc_url(site_url() ) . '<br><br>';

			if ( ! empty( $failed_smtp ) ) {
				if ( in_array( 'bvopt_smpt_1', $failed_smtp ) ) {
					$bvopt_smpt_1 = get_option( 'bvopt_smpt_alt_setting_1' );

					$message .= 'SMTP Settings 1 looks also down...<br>';
					$message .= 'Please check below SMTP Settings 1 details...<br>';

					$message .= '<br> smtp host: ';
					$message .= ! empty( $bvopt_smpt_1['smtp_host'] ) ? $bvopt_smpt_1['smtp_host'] : '';
					$message .= '<br> smtp username: ';
					$message .= ! empty( $bvopt_smpt_1['smtp_username'] ) ? $bvopt_smpt_1['smtp_username'] : '';
					// $message .= "<br> smtp encryption: ";
					// $message .= ! empty( $bvopt_smpt_1['smtp_encryption'] ) ? $bvopt_smpt_1['smtp_encryption'] : '';
					// $message .= "<br> smtp port: ";
					// $message .= ! empty( $bvopt_smpt_1['smtp_port'] ) ? $bvopt_smpt_1['smtp_port'] : '';
					$message .= '<br> smtp from: ';
					$message .= ! empty( $bvopt_smpt_1['smtp_from'] ) ? $bvopt_smpt_1['smtp_from'] : '';
					$message .= '<br> smtp from name: ';
					$message .= ! empty( $bvopt_smpt_1['smtp_from_name'] ) ? $bvopt_smpt_1['smtp_from_name'] : '';
					$message .= '<br><br>';
				}

				if ( in_array( 'bvopt_smpt_2', $failed_smtp ) ) {
					$bvopt_smpt_2 = get_option( 'bvsmtp_smpt_alt_setting_2' );

					$message .= 'SMTP Settings 2 looks also down...<br>';
					$message .= 'Please check below SMTP Settings 2 details...<br>';

					$message .= '<br> smtp host: ';
					$message .= ! empty( $bvopt_smpt_2['smtp_host'] ) ? $bvopt_smpt_2['smtp_host'] : '';
					$message .= '<br> smtp username: ';
					$message .= ! empty( $bvopt_smpt_2['smtp_username'] ) ? $bvopt_smpt_2['smtp_username'] : '';
					// $message .= "<br> smtp encryption: ";
					// $message .= ! empty( $bvopt_smpt_2['smtp_encryption'] ) ? $bvopt_smpt_2['smtp_encryption'] : '';
					// $message .= "<br> smtp port: ";
					// $message .= ! empty( $bvopt_smpt_2['smtp_port'] ) ? $bvopt_smpt_2['smtp_port'] : '';
					$message .= '<br> smtp from: ';
					$message .= ! empty( $bvopt_smpt_2['smtp_from'] ) ? $bvopt_smpt_2['smtp_from'] : '';
					$message .= '<br> smtp from name: ';
					$message .= ! empty( $bvopt_smpt_2['smtp_from_name'] ) ? $bvopt_smpt_2['smtp_from_name'] : '';
					$message .= '<br><br>';
				}
			}

			if ( function_exists( 'bvsmtp_send_mail_using_custom_smtp' ) ) {
				$send_to = explode( ',', $smtp_downtime_sendmailto );
				$res = bvsmtp_send_mail_using_phpmailer( $working_smtp, $send_to, $subject, $message );

				if ( $res['success'] ) {
					$bvsmtp_smpt_global_settings['smtp_downtime_last_send_datetime'] = gmdate( 'Y-m-d H:i:s' );
					update_option( 'bvsmtp_smpt_global_settings', $bvsmtp_smpt_global_settings );
				}
			}
		}
	}
}

/**
 * Send mail using alternate smtp settings.
 *
 * @param array  $send_to Email addresses to send the mail.
 * @param string $subject Email subject.
 * @param string $messages Email body.
 * @return true
 */
function bvsmtp_send_mail_using_custom_smtp( $send_to, $subject, $messages ) {
	if ( empty( $send_to ) || empty( $subject ) || empty( $messages ) ) {
		return false;
	}

	$bvopt_smpt_1 = get_option( 'bvopt_smpt_alt_setting_1' );
	$bvopt_smpt_2 = get_option( 'bvsmtp_smpt_alt_setting_2' );
	$bvopt_smpt_3 = get_option( 'bvsmtp_smpt_alt_setting_3' );

	$smtp_settings = array(
		'bvopt_smpt_1' => $bvopt_smpt_1,
		'bvopt_smpt_2' => $bvopt_smpt_2,
		'bvopt_smpt_3' => $bvopt_smpt_3,
	);

	$failed_smtp = array();

	foreach ( $smtp_settings as $key => $value ) {
		if ( 'yes' == $value['smtp_isactive'] ) {

				$res = bvsmtp_send_mail_using_phpmailer( $value, $send_to, $subject, $messages );

			if ( $res['success'] ) {

				$working_smtp = $smtp_settings[ $key ];

				if ( function_exists( 'bvsmtp_send_default_admin_mail_about_smtp_down' ) ) {
					bvsmtp_send_default_admin_mail_about_smtp_down( $working_smtp, $failed_smtp );
				}
				break;
			} else {
				$failed_smtp[] = $key;
			}
		}
	}

	return true;
}

/**
 * Send mail using php mailer.
 *
 * @param array  $smtp_setting The SMTP stored setting used for send smtp mail.
 * @param array  $send_to Email addresses to send the mail.
 * @param string $subject Email subject.
 * @param string $messages Email body.
 * @return array
 */
function bvsmtp_send_mail_using_phpmailer( $smtp_setting, $send_to, $subject, $messages ) {
	$bvopt_val_smtp_host = ! empty( $smtp_setting['smtp_host'] ) ? sanitize_text_field( $smtp_setting['smtp_host'] ) : '';
	$bvopt_val_smtp_username = ! empty( $smtp_setting['smtp_username'] ) ? sanitize_text_field( $smtp_setting['smtp_username'] ) : '';
	$bvopt_val_smtp_password = ! empty( $smtp_setting['smtp_password'] ) ? sanitize_text_field( $smtp_setting['smtp_password'] ) : '';
	$bvopt_val_smtp_encryption = ! empty( $smtp_setting['smtp_encryption'] ) ? sanitize_text_field( $smtp_setting['smtp_encryption'] ) : '';
	$bvopt_val_smtp_port = ! empty( $smtp_setting['smtp_port'] ) ? intval( $smtp_setting['smtp_port'] ) : '';
	$bvopt_val_smtp_from = ! empty( $smtp_setting['smtp_from'] ) ? sanitize_email( $smtp_setting['smtp_from'] ) : '';
	$bvopt_val_smtp_from_name = ! empty( $smtp_setting['smtp_from_name'] ) ? sanitize_text_field( $smtp_setting['smtp_from_name'] ) : '';

	if ( ! class_exists( 'PHPMailer\PHPMailer\PHPMailer' ) ) {
		return array(
			'success' => false,
			'message' => 'The Class PHPMailer is not found.',
		);
	}

		$mail = new PHPMailer(); // Passing `true` enables exceptions.

		$mail->isSMTP();
		$mail->Host = $bvopt_val_smtp_host;
		$mail->SMTPAuth = true; // Enable SMTP authentication.
		$mail->Username = $bvopt_val_smtp_username;
		$mail->Password = bvsmtp_decrypt( $bvopt_val_smtp_password );
		$mail->SMTPSecure = $bvopt_val_smtp_encryption;
		$mail->Port = $bvopt_val_smtp_port;

		$mail->setFrom( $bvopt_val_smtp_from, $bvopt_val_smtp_from_name );

	if ( ! empty( $send_to ) ) {
		foreach ( $send_to as $to ) {
			$mail->addAddress( sanitize_email( $to ) );
		}
	}

		$mail->isHTML( true );   // Set email format to HTML.
		$mail->Subject = sanitize_text_field( $subject );
		$mail->Body = wp_kses_post( $messages );

	if ( $mail->send() ) {
		return array(
			'success' => true,
			'message' => 'The mail is sent successfully.',
		);
	} else {
		return array(
			'success' => false,
			'message' => 'The mail could not be sent due to some errors!',
		);
	}
}
