<?php
/**
 * File for creating admin pages.
 *
 * @package alternate-smtp
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Adds the admin menu page Alternate SMTP Settings.
 *
 * @return void
 */
function bvsmtp_add_admin_menu() {
	add_menu_page(
		esc_html__( 'Alternate SMTP | Settings', 'alternate-smtp' ), // Page Title.
		esc_html__( 'Alternate SMTP', 'alternate-smtp' ), // Link text.
		'manage_options', // User capabilities.
		'bvsmtp_alternate_smtp_settings', // Page slug.
		'bvsmtp_alternate_smtp_settings', // This function prints the page content.
		'dashicons-email', // Icon (from Dashicons for example).
		14 // Menu position.
	);

	add_submenu_page( 'bvsmtp_alternate_smtp_settings', esc_html__( 'Alternate SMTP | Help Page', 'alternate-smtp' ), 'Help', 'read', 'bv_alternate_smtp_help', 'bvsmtp_help_page' );
}
add_action( 'admin_menu', 'bvsmtp_add_admin_menu' );

/**
 * Display the content for the admin smtp setting page.
 *
 * @return void
 */
function bvsmtp_alternate_smtp_settings() {
	 // must check that the user has the required capability.
	if ( ! current_user_can( 'manage_options' ) ) {
		wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'alternate-smtp' ) );
	}

	// SMTP form fields.
	$bv_smtp_submit_btn = 'submit_smtp_settings';
	$bv_smtp_setting_index = 'smtp_setting_index';
	$bv_smtp_isactive = 'bv_smtp_isactive';
	$bv_smtp_host = 'smtp_host';
	$bv_smtp_username = 'smtp_username';
	$bv_smtp_password = 'smtp_password';
	$bv_smtp_port = 'smtp_port';
	$bv_smtp_encryption = 'smtp_encryption';
	$bv_smtp_from = 'smtp_from';
	$bv_smtp_from_name = 'smtp_from_name';
	$bv_smtp_downtime_sendmailto = 'smtp_downtime_sendmailto';

	$bvopt_smpt_1 = get_option( 'bvsmtp_smpt_alt_setting_1' );
	$bvopt_smpt_2 = get_option( 'bvsmtp_smpt_alt_setting_2' );
	$bvopt_smpt_3 = get_option( 'bvsmtp_smpt_alt_setting_3' );

	// For global settings.
	$bvopt_smpt_global_settings = get_option( 'bvsmtp_smpt_global_settings' );

	// Now display the settings editing screen.
	echo '<div class="wrap">';
	// Header.
	echo '<h2>' . esc_html__( 'Alternate SMTP Settings', 'alternate-smtp' ) . '</h2>';

	include __DIR__ . '/bvsmtp-forms.php';

	?>

	</div>

	<?php
}

/**
 * Display the content for the admin help page.
 *
 * @return void
 */
function bvsmtp_help_page() {
	include 'html/help-page.html';
}

/**
 * Alter submenu link text.
 *
 * @return void
 */
function bvsmtp_alter_menu_link_text() {
	global $submenu;

	// Check if our main menu exists.
    if (isset($submenu['bvsmtp_alternate_smtp_settings'])) {
		$submenu['bvsmtp_alternate_smtp_settings'][0][0] = 'Settings';
	}
}
 add_action( 'admin_menu', 'bvsmtp_alter_menu_link_text' );
