<?php
/**
 * File for handling ajax requests.
 *
 * @package alternate-smtp
 */

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

/**
 * Save SMTP Settings.
 *
 * @return void
 */
function bvsmtp_save_settings() {

	$nonce = ! empty( $_POST['nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['nonce'] ) ) : '';

	// Check for nonce security.
	if ( empty( $nonce ) || ! wp_verify_nonce( $nonce, 'ajax-nonce' )) {
		wp_send_json_error( 'Invalid Request!' );
	}

	$bvopt_val_smtp_setting_index = ! empty($_POST['smtp_setting_index']) ? sanitize_text_field( wp_unslash( $_POST['smtp_setting_index'] ) ) : '';
	$bvopt_val_smtp_isactive = ! empty($_POST['bv_smtp_isactive']) ? sanitize_text_field( wp_unslash( $_POST[ 'bv_smtp_isactive' ] ) ) : '' ;
	$bvopt_val_smtp_host = ! empty($_POST['smtp_host']) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_host' ] ) ) : '' ;
	$bvopt_val_smtp_username = ! empty($_POST['smtp_username']) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_username' ] ) ) : '' ;
	$bvopt_val_smtp_password = ! empty( $_POST[ 'smtp_password' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_password' ] ) ) : '';
	$bvopt_val_smtp_port = ! empty( $_POST[ 'smtp_port' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_port' ] ) ) : '';
	$bvopt_val_smtp_encryption = ! empty( $_POST[ 'smtp_encryption' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_encryption' ] ) ) : '';
	$bvopt_val_smtp_from = ! empty( $_POST[ 'smtp_from' ] ) ? sanitize_email( wp_unslash( $_POST[ 'smtp_from' ] ) ) : '';
	$bvopt_val_smtp_from_name = ! empty( $_POST[ 'smtp_from_name' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_from_name' ] ) ) : '';

	$bv_smtp_alt_setting = array(
		'smtp_isactive' => $bvopt_val_smtp_isactive,
		'smtp_host' => $bvopt_val_smtp_host,
		'smtp_username' => $bvopt_val_smtp_username,
		'smtp_password' => $bvopt_val_smtp_password,
		'smtp_port' => $bvopt_val_smtp_port,
		'smtp_encryption' => $bvopt_val_smtp_encryption,
		'smtp_from' => $bvopt_val_smtp_from,
		'smtp_from_name' => $bvopt_val_smtp_from_name,
	);

	// Save the posted value in the database.
	switch ( $bvopt_val_smtp_setting_index ) {
		case '1':
			// code...
			$old_setting = get_option( 'bvsmtp_smpt_alt_setting_1' );
			if ( ! empty( $bvopt_val_smtp_password ) ) {
				$bv_smtp_alt_setting['smtp_password'] = bvsmtp_encrypt( $bvopt_val_smtp_password );
			}
			if ( ! empty( $old_setting['smtp_password'] ) && empty( $bvopt_val_smtp_password ) ) {
				$bv_smtp_alt_setting['smtp_password'] = $old_setting['smtp_password'];
			}
			if ( $old_setting === $bv_smtp_alt_setting ) {
				$res = true;
			} else {
				$res = update_option( 'bvsmtp_smpt_alt_setting_1', $bv_smtp_alt_setting );
			}
			break;
		case '2':
			// code...
			$old_setting = get_option( 'bvsmtp_smpt_alt_setting_2' );
			if ( ! empty( $bvopt_val_smtp_password ) ) {
				$bv_smtp_alt_setting['smtp_password'] = bvsmtp_encrypt( $bvopt_val_smtp_password );
			}
			if ( ! empty( $old_setting['smtp_password'] ) && empty( $bvopt_val_smtp_password ) ) {
				$bv_smtp_alt_setting['smtp_password'] = $old_setting['smtp_password'];
			}
			if ( $old_setting === $bv_smtp_alt_setting ) {
				$res = true;
			} else {
				$res = update_option( 'bvsmtp_smpt_alt_setting_2', $bv_smtp_alt_setting );
			}
			break;
		case '3':
			// code...
			$old_setting = get_option( 'bvsmtp_smpt_alt_setting_3' );
			if ( ! empty( $bvopt_val_smtp_password ) ) {
				$bv_smtp_alt_setting['smtp_password'] = bvsmtp_encrypt( $bvopt_val_smtp_password );
			}
			if ( ! empty( $old_setting['smtp_password'] ) && empty( $bvopt_val_smtp_password ) ) {
				$bv_smtp_alt_setting['smtp_password'] = $old_setting['smtp_password'];
			}
			if ( $old_setting === $bv_smtp_alt_setting ) {
				$res = true;
			} else {
				$res = update_option( 'bvsmtp_smpt_alt_setting_3', $bv_smtp_alt_setting );
			}
			break;
		default:
			// code...
			break;
	}
	
	if ( $res ) {
		wp_send_json_success( 'SMTP Setting ' . $bvopt_val_smtp_setting_index . ' is saved successfully.' );
	} else {
		wp_send_json_error( 'Error! error occurred while saving the SMTP Settings!' );
	}
}
add_action( 'wp_ajax_bvsmtp_save_settings', 'bvsmtp_save_settings' );

/**
 * Test SMTP Setting by sending a test mail.
 *
 * @return void
 */
function bvsmtp_test_settings() {

	

	$nonce = ! empty( $_POST['nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['nonce'] ) ) : '';

	// Check for nonce security.
	if ( empty( $nonce ) || ! wp_verify_nonce( $nonce, 'ajax-nonce' ) ) {
		wp_send_json_error( 'Invalid Request!' );
	}


	//parse_str( wp_unslash( $_POST['form_data'] ), $form_data );

	$smtp_setting_selected = ! empty( $_POST[ 'smtp_setting' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'smtp_setting' ] ) ) : '';
	$send_to =  ! empty( $_POST[ 'test_email_send_to' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'test_email_send_to' ] ) ) : '' ;
	$subject = ! empty( $_POST[ 'test_email_subject' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'test_email_subject' ] ) ) : 'Test SMTP Mail';
	$message = 'This is a test mail send by sender. no need to reply or take any action.';

	

	// Save the posted value in the database.
	switch ( $smtp_setting_selected ) {
		case 'smtp_setting_1':
			// code...
			$smtp_setting = get_option( 'bvsmtp_smpt_alt_setting_1' );
			break;
		case 'smtp_setting_2':
			// code...
			$smtp_setting = get_option( 'bvsmtp_smpt_alt_setting_2' );
			break;
		case 'smtp_setting_3':
			// code...
			$smtp_setting = get_option( 'bvsmtp_smpt_alt_setting_3' );
			break;
		default:
			// code...
			wp_send_json_error( 'Invalid Request!' );
			break;
	}

	$res = bvsmtp_send_test_mail_using_custom_smtp( array( $send_to ), $subject, $message, $smtp_setting );

	$res = html_entity_decode( esc_html( $res ) );
	
	if ( strpos( $res, 'Message has been sent' ) != false ) {
		wp_send_json_success( $res );
	} else {
		wp_send_json_error( $res );
	}
}
add_action( 'wp_ajax_bvsmtp_test_settings', 'bvsmtp_test_settings' );

/**
 * Send test mail using selected smtp settings.
 *
 * @param array  $send_to Email addresses to send the mail.
 * @param string $subject Email subject.
 * @param string $messages Email body.
 * @param string $smtp_setting Selected SMTP Setting.
 * @return true
 */
function bvsmtp_send_test_mail_using_custom_smtp( $send_to, $subject, $messages, $smtp_setting ) {
		$bvopt_val_smtp_host = ! empty( $smtp_setting['smtp_host'] ) ? $smtp_setting['smtp_host'] : '';
		$bvopt_val_smtp_username = ! empty( $smtp_setting['smtp_username'] ) ? $smtp_setting['smtp_username'] : '';
		$bvopt_val_smtp_password = ! empty( $smtp_setting['smtp_password'] ) ? $smtp_setting['smtp_password'] : '';
		$bvopt_val_smtp_encryption = ! empty( $smtp_setting['smtp_encryption'] ) ? $smtp_setting['smtp_encryption'] : '';
		$bvopt_val_smtp_port = ! empty( $smtp_setting['smtp_port'] ) ? $smtp_setting['smtp_port'] : '';
		$bvopt_val_smtp_from = ! empty( $smtp_setting['smtp_from'] ) ? $smtp_setting['smtp_from'] : '';
		$bvopt_val_smtp_from_name = ! empty( $smtp_setting['smtp_from_name'] ) ? $smtp_setting['smtp_from_name'] : '';

		$mail = new PHPMailer( true ); // passing `true` enables exceptions.

		ob_start();

	try {
		$mail->isSMTP();
		$mail->Host = $bvopt_val_smtp_host;
		$mail->SMTPAuth = true; // Enable SMTP authentication.
		$mail->Username = $bvopt_val_smtp_username;
		$mail->Password = bvsmtp_decrypt( $bvopt_val_smtp_password );
		$mail->SMTPSecure = $bvopt_val_smtp_encryption;
		$mail->Port = $bvopt_val_smtp_port;

		// Enable SMTP debugging (2 for client and server messages).
		$mail->SMTPDebug = 2;

		$mail->setFrom( $bvopt_val_smtp_from, $bvopt_val_smtp_from_name );

		if ( ! empty( $send_to ) ) {
			foreach ( $send_to as $to ) {
				$mail->addAddress( $to );
			}
		}

		$mail->isHTML( true );
		$mail->Subject = $subject;
		$mail->Body = $messages;
		$res = $mail->send();
		echo '<br>Message has been sent.';
	} catch ( Exception $e ) {
		echo '<br>Message could not be sent.';
	}

		return ob_get_clean();
}

/**
 * To save smtp global settings.
 *
 * @return void
 */
function bvsmtp_save_smtp_global_settings() {
	$nonce = ! empty( $_POST['nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['nonce'] ) ) : '';

	// Check for nonce security.
	if ( empty( $nonce ) || ! wp_verify_nonce( $nonce, 'ajax-nonce' ) ) {
		wp_send_json_error( 'Invalid Request!' );
	}

	//parse_str( wp_unslash( $_POST['form_data'] ), $form_data );

	// Option Table Key Field for global settings.
	$bvopt_smpt_global_settings = 'bvsmtp_smpt_global_settings';

	// Form Field Values.
	$bv_val_smtp_downtime_sendmailto = ! empty($_POST['smtp_downtime_sendmailto']) ? sanitize_text_field( wp_unslash( $_POST['smtp_downtime_sendmailto'] ) ) : '';
	$bv_val_smtp_downtime_status = ! empty($_POST['smtp_downtime_status']) ? sanitize_text_field(wp_unslash($_POST['smtp_downtime_status'])) : '' ;
	$bv_val_smtp_downtime_interval = ! empty($_POST['smtp_downtime_interval']) ? sanitize_text_field(wp_unslash($_POST['smtp_downtime_interval'])) : '';
	$bv_val_smtp_downtime_last_send_datetime = ! empty($_POST['smtp_downtime_last_send_datetime']) ? sanitize_text_field(wp_unslash($_POST['smtp_downtime_last_send_datetime'])) : '' ;

	$bvopt_val_smpt_global_settings = array(
		'smtp_downtime_sendmailto' => $bv_val_smtp_downtime_sendmailto,
		'smtp_downtime_status'=> $bv_val_smtp_downtime_status,
		'smtp_downtime_interval' => $bv_val_smtp_downtime_interval,
		'smtp_downtime_last_send_datetime' => $bv_val_smtp_downtime_last_send_datetime,
	);

	$old_setting = get_option( $bvopt_smpt_global_settings );
	if ( $old_setting === $bvopt_val_smpt_global_settings ) {
		$res = true;
	} else {
		$res = update_option( $bvopt_smpt_global_settings, $bvopt_val_smpt_global_settings );
	}

	if ( $res ) {
		wp_send_json_success( 'SMTP global settings is saved successfully.' );
	} else {
		wp_send_json_error( 'Error! error occurred while saving the SMTP global settings!' );
	}
}
add_action( 'wp_ajax_bvsmtp_save_smtp_global_settings', 'bvsmtp_save_smtp_global_settings' );
